<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Helper\Product\Type;

use Magento\Bundle\Model\OptionRepository;
use Magento\Bundle\Model\ResourceModel\Selection;
use Magento\Catalog\Model\Product;
use Magento\Catalog\Model\Product\Option;
use Magento\CatalogInventory\Api\StockRegistryInterface;
use Magento\CatalogInventory\Model\Spi\StockStateProviderInterface;
use Magento\Framework\DataObject;
use Magento\Quote\Model\Quote\Item as QuoteItem;
use Shopgate\Base\Model\Shopgate\Extended\OrderItem;

class Bundle extends Generic
{
    /** @var OptionRepository */
    private $bundleOptions;
    /** @var Selection\CollectionFactory */
    private $selectionCollection;

    /**
     * @param Selection\CollectionFactory $selectCollection
     * @param Option\Repository           $productOptionRepo
     * @param OptionRepository            $bundleOptions
     * @param StockRegistryInterface      $stockRegistry
     * @param StockStateProviderInterface $stateProvider
     */
    public function __construct(
        Selection\CollectionFactory $selectCollection,
        Option\Repository $productOptionRepo,
        OptionRepository $bundleOptions,
        StockRegistryInterface $stockRegistry,
        StockStateProviderInterface $stateProvider
    ) {
        $this->bundleOptions       = $bundleOptions;
        $this->selectionCollection = $selectCollection;
        parent::__construct($productOptionRepo, $stockRegistry, $stateProvider);
    }

    /**
     * @inheritdoc
     */
    public function parseOptions($product)
    {
        if (!$this->getItem() instanceof OrderItem) {
            throw new Exception('The item provided is supposed to be of type OrderItem in class: ' . get_class($this));
        }

        $data = [];
        foreach ($this->getItem()->getOptions() as $orderOption) {
            /* @var $orderOption \ShopgateOrderItemOption */
            $optionId = $orderOption->getOptionNumber();
            $value    = $orderOption->getValueNumber();

            $productOption = $this->getOptionById($product->getSku(), $optionId);
            //todo-sg: need to replicate @see https://shopgate.atlassian.net/browse/MAGENTO-820
            if (!$productOption->getOptionId()
                /*|| ($productOption->getOptionId() && $productOption->getParentId() != $product->getId())*/
            ) {
                $productOption = $this->getProductOptionById($product->getSku(), $optionId);

                if ($this->isHierarchyType($productOption)) {
                    if ($value == 0) {
                        continue;
                    }
                    $value = [$value];
                }
                $data['options'][$optionId] = $value;
            } else {
                if ($this->isHierarchyType($productOption)) {
                    if (!$value) {
                        continue;
                    }
                    $value = [$value];
                }

                $bundleSelection = $this->getSelectionById($value);

                $data['bundle_option_qty'][$optionId] = max(1, (int) $bundleSelection->getSelectionQty());
                $data['bundle_option'][$optionId]     = $value;
            }
        }

        return $data;
    }

    /**
     * @param string       $sku
     * @param string | int $optionId
     *
     * @return \Magento\Bundle\Api\Data\OptionInterface
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getOptionById($sku, $optionId)
    {
        return $this->bundleOptions->get($sku, $optionId);
    }

    /**
     * @param string | int $selectionId
     *
     * @return Product | \Magento\Bundle\Model\Selection | DataObject - joined return of both
     */
    public function getSelectionById($selectionId)
    {
        $collection = $this->selectionCollection->create()->setSelectionIdsFilter([$selectionId]);

        return $collection->getFirstItem();
    }

    /**
     * @return array
     * @throws Exception
     */
    public function getStockData()
    {
        if (!$this->getItem() instanceof QuoteItem) {
            throw new Exception('The item provided is supposed to be of type QuoteItem in class: ' . get_class($this));
        }

        $product    = $this->getItem()->getProduct();
        $stockItem  = $this->stockRegistry->getStockItem($product->getId(), $product->getStoreId());
        $errors     = [];
        $isBuyable  = true;
        $qtyBuyable = null;

        foreach ($this->getItem()->getChildren() as $childItem) {
            $childProduct    = $childItem->getProduct();
            $childStock      = $this->stockRegistry->getStockItem($childProduct->getId(), $childProduct->getStoreId());
            $checkIncrements = $this->stateProvider->checkQtyIncrements($childStock, $childItem->getQty());

            if ($childStock->getManageStock()
                && !$childProduct->isSaleable()
                && !$childStock->getBackorders()
            ) {
                $isBuyable        = false;
                $error['type']    = \ShopgateLibraryException::CART_ITEM_OUT_OF_STOCK;
                $error['message'] = \ShopgateLibraryException::getMessageFor(
                    \ShopgateLibraryException::CART_ITEM_OUT_OF_STOCK
                );
                $errors[]         = $error;
            } elseif ($childStock->getManageStock()
                && !$this->stateProvider->checkQty($childStock, $childItem->getQty())
                && !$childStock->getBackorders()
            ) {
                $isBuyable        = false;
                $error['type']    = \ShopgateLibraryException::CART_ITEM_REQUESTED_QUANTITY_NOT_AVAILABLE;
                $error['message'] = \ShopgateLibraryException::getMessageFor(
                    \ShopgateLibraryException::CART_ITEM_REQUESTED_QUANTITY_NOT_AVAILABLE
                );
                $errors[]         = $error;
                if ($qtyBuyable == null || $qtyBuyable > $childStock->getQty()) {
                    $qtyBuyable = $childStock->getQty();
                }
            } elseif ($childStock->getManageStock() && $checkIncrements->getData('has_error')) {
                $isBuyable        = false;
                $error['type']    = \ShopgateLibraryException::CART_ITEM_REQUESTED_QUANTITY_NOT_AVAILABLE;
                $error['message'] = \ShopgateLibraryException::getMessageFor(
                    \ShopgateLibraryException::CART_ITEM_REQUESTED_QUANTITY_NOT_AVAILABLE
                );
                $errors[]         = $error;
                $stockItem->setQty(
                    (int) ($this->getItem()->getQtyToAdd() / $stockItem->getQtyIncrements())
                    * $stockItem->getQtyIncrements()
                );
            }
        }

        $qtyBuyable = (int) ($qtyBuyable !== null) ? $qtyBuyable : $this->getItem()->getQty();
        $error      = array_pop($errors);

        return [
            'is_buyable'     => (int) $isBuyable,
            'qty_buyable'    => $qtyBuyable,
            'stock_quantity' => (int) $stockItem->getQty(),
            'error'          => isset($error['type']) ? $error['type'] : null,
            'error_text'     => isset($error['message']) ? $error['message'] : null,
        ];
    }
}
