<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Helper\Product\Type;

use Magento\Bundle\Api\Data\OptionInterface;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Catalog\Model\Product as MageProduct;
use Magento\Catalog\Model\Product\Option;
use Magento\CatalogInventory\Api\StockRegistryInterface;
use Magento\CatalogInventory\Model\Spi\StockStateProviderInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Quote\Model\Quote\Item as QuoteItem;
use Shopgate\Base\Model\Shopgate\Extended\OrderItem;

class Generic
{

    /** @var StockRegistryInterface */
    protected $stockRegistry;
    /** @var StockStateProviderInterface */
    protected $stateProvider;
    /** @var Option\Repository */
    private $productOptionRepo;
    /** @var ProductRepositoryInterface */
    protected $productRepository;
    /** @var OrderItem | MageProduct | QuoteItem $item */
    protected $item;

    /**
     * @param Option\Repository           $productOptionRepo
     * @param StockRegistryInterface      $stockRegistry
     * @param StockStateProviderInterface $stateProvider
     * @param ProductRepositoryInterface  $productRepository
     */
    public function __construct(
        Option\Repository $productOptionRepo,
        StockRegistryInterface $stockRegistry,
        StockStateProviderInterface $stateProvider,
        ProductRepositoryInterface $productRepository
    ) {
        $this->productOptionRepo = $productOptionRepo;
        $this->stockRegistry     = $stockRegistry;
        $this->stateProvider     = $stateProvider;
        $this->productRepository = $productRepository;

    }

    /**
     * todo-sg: move options into their own class
     *
     * @param MageProduct | null $product
     *
     * @return array
     * @throws Exception
     * @throws NoSuchEntityException
     */
    public function parseOptions($product)
    {
        if (!$this->getItem() instanceof OrderItem) {
            throw new Exception('The item provided is supposed to be of type OrderItem in class: ' . get_class($this));
        }

        $data = [];
        foreach ($this->getItem()->getOptions() as $orderOption) {
            /* @var $orderOption \ShopgateOrderItemOption */
            $optionId = $orderOption->getOptionNumber();
            $value    = $orderOption->getValueNumber();

            $productOption = $this->getProductOptionById($product->getSku(), $optionId);
            if ($this->isHierarchyType($productOption)) {
                if ($value == 0) {
                    continue;
                }
                $value = [$value];
            }
            $data['options'][$optionId] = $value;
        }

        return $data;
    }

    /**
     * @param string       $sku
     * @param string | int $optionId
     *
     * @return OptionInterface
     * @throws NoSuchEntityException
     */
    public function getProductOptionById($sku, $optionId)
    {
        return $this->productOptionRepo->get($sku, $optionId);
    }

    /**
     * @param OptionInterface $option
     *
     * @return bool
     */
    protected function isHierarchyType(OptionInterface $option)
    {
        $type = $option->getType();

        return $type === Option::OPTION_TYPE_CHECKBOX || $type === Option::OPTION_TYPE_MULTIPLE;
    }

    /**
     * @return array
     * @throws Exception
     */
    public function getStockData()
    {
        if (!$this->getItem() instanceof QuoteItem) {
            throw new Exception('The item provided is supposed to be of type QuoteItem in class: ' . get_class($this));
        }

        $errors          = [];
        $isBuyable       = true;
        $product         = $this->getItem()->getProduct();
        $stockItem       = $this->stockRegistry->getStockItem($product->getId(), $product->getStoreId());
        $checkIncrements = $this->stateProvider->checkQtyIncrements($stockItem, $this->getItem()->getQty());

        if ($stockItem->getManageStock() && !$product->isSaleable() && !$stockItem->getBackorders()) {
            $isBuyable        = false;
            $error['type']    = \ShopgateLibraryException::CART_ITEM_OUT_OF_STOCK;
            $error['message'] = \ShopgateLibraryException::getMessageFor(
                \ShopgateLibraryException::CART_ITEM_OUT_OF_STOCK
            );
            $errors[]         = $error;
        } elseif ($stockItem->getManageStock()
            && !$this->stateProvider->checkQty($stockItem, $this->getItem()->getQty())
            && !$stockItem->getBackorders()
        ) {
            $isBuyable        = false;
            $error['type']    = \ShopgateLibraryException::CART_ITEM_REQUESTED_QUANTITY_NOT_AVAILABLE;
            $error['message'] = \ShopgateLibraryException::getMessageFor(
                \ShopgateLibraryException::CART_ITEM_REQUESTED_QUANTITY_NOT_AVAILABLE
            );
            $errors[]         = $error;
        } elseif ($stockItem->getManageStock() && $checkIncrements->getHasError()) {
            $isBuyable        = false;
            $error            = [];
            $error['type']    = \ShopgateLibraryException::CART_ITEM_REQUESTED_QUANTITY_NOT_AVAILABLE;
            $error['message'] = \ShopgateLibraryException::getMessageFor(
                \ShopgateLibraryException::CART_ITEM_REQUESTED_QUANTITY_NOT_AVAILABLE
            );
            $errors[]         = $error;
            $stockItem->setQty(
                (int)($this->getItem()->getQtyToAdd() / $stockItem->getQtyIncrements())
                * $stockItem->getQtyIncrements()
            );
        }
        $qtyBuyable = $isBuyable ? (int)$this->getItem()->getQty() : (int)$stockItem->getQty();
        $error      = array_pop($errors);

        return [
            'is_buyable'     => (int)$isBuyable,
            'qty_buyable'    => $qtyBuyable,
            'stock_quantity' => (int)$stockItem->getQty(),
            'error'          => isset($error['type']) ? $error['type'] : null,
            'error_text'     => isset($error['message']) ? $error['message'] : null,
        ];
    }

    /**
     * @param OrderItem | MageProduct | QuoteItem $item $item
     *
     * @return $this
     */
    public function setItem($item)
    {
        $this->item = $item;

        return $this;
    }

    /**
     * @return MageProduct | QuoteItem | OrderItem
     */
    public function getItem()
    {
        return $this->item;
    }

    /**
     * @return \Magento\Catalog\Model\Product[]
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getChildren()
    {
        return [];
    }
}
