<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Helper;

use Magento\Catalog\Model\Product as MageProduct;
use Magento\ConfigurableProduct\Model\Product\Type\Configurable;
use Magento\Framework\Api\SimpleDataObjectConverter;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Phrase;
use Magento\Framework\Registry;
use Magento\Framework\Webapi\Exception;
use Magento\GroupedProduct\Model\Product\Type\Grouped;
use Magento\Quote\Model\Quote as MageQuote;
use Magento\Quote\Model\QuoteRepository;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Tax\Helper\Data as Tax;
use Shopgate\Base\Helper\Product\Utility;
use Shopgate\Base\Helper\Quote\Coupon;
use Shopgate\Base\Model\Shopgate\Extended;
use Shopgate\Base\Model\Utility\SgLoggerInterface;
use ShopgateLibraryException;

/**
 * This class must not return anything except itself as it only
 * handles quote loaders
 */
class Quote
{

    /** @var MageQuote */
    protected $quote;
    /** @var Extended\Base */
    protected $sgBase;
    /** @var SgLoggerInterface */
    protected $log;
    /** @var Utility */
    protected $productHelper;
    /** @var Tax */
    protected $taxData;
    /** @var Quote\Customer */
    protected $quoteCustomer;
    /** @var Registry */
    protected $coreRegistry;
    /** @var StoreManagerInterface */
    protected $storeManager;
    /** @var Coupon */
    private $quoteCouponHelper;
    /** @var QuoteRepository */
    private $quoteRepository;

    /**
     * @param MageQuote             $quote
     * @param Extended\Base         $cart
     * @param SgLoggerInterface     $logger
     * @param Utility               $productHelper
     * @param Tax                   $taxData
     * @param Quote\Customer        $quoteCustomer
     * @param Registry              $coreRegistry
     * @param StoreManagerInterface $storeManager
     * @param Coupon                $quoteCoupon
     * @param QuoteRepository       $quoteRepository
     */
    public function __construct(
        MageQuote $quote,
        Extended\Base $cart,
        SgLoggerInterface $logger,
        Utility $productHelper,
        Tax $taxData,
        Quote\Customer $quoteCustomer,
        Registry $coreRegistry,
        StoreManagerInterface $storeManager,
        Coupon $quoteCoupon,
        QuoteRepository $quoteRepository
    ) {
        $this->quote             = $quote;
        $this->sgBase            = $cart;
        $this->log               = $logger;
        $this->productHelper     = $productHelper;
        $this->taxData           = $taxData;
        $this->quoteCustomer     = $quoteCustomer;
        $this->coreRegistry      = $coreRegistry;
        $this->storeManager      = $storeManager;
        $this->quoteCouponHelper = $quoteCoupon;
        $this->quoteRepository   = $quoteRepository;
    }

    /**
     * @param array $methods - quote methods to load
     *
     * @return MageQuote
     */
    public function load(array $methods)
    {
        $this->quote->setStoreId($this->storeManager->getStore()->getId());

        foreach ($methods as $rawMethod) {
            $method = 'set' . SimpleDataObjectConverter::snakeCaseToUpperCamelCase($rawMethod);
            $this->log->debug('Starting method ' . $method);
            $this->{$method}();
            $this->log->debug('Finished method ' . $method);
        }

        return $this->quote;
    }

    /**
     * Clean up quote table
     */
    public function cleanup()
    {
        $this->quote->delete();
    }

    /**
     * Assign coupons to the quote
     */
    protected function setExternalCoupons()
    {
        $this->quoteCouponHelper->setCoupon();
    }

    /**
     * Assigns Shopgate cart items to quote
     */
    protected function setItems()
    {
        foreach ($this->sgBase->getItems() as $item) {
            if ($item->isSgCoupon()) {
                continue;
            }

            $info        = $item->getInternalOrderInfo();
            $amountNet   = $item->getUnitAmount();
            $amountGross = $item->getUnitAmountWithTax();
            $product     = $this->productHelper->loadById($info->getProductId());

            if (!$product->getId()) {
                throw new ShopgateLibraryException(
                    ShopgateLibraryException::CART_ITEM_PRODUCT_NOT_FOUND,
                    'Product ID: ' . $info->getProductId(),
                    true
                );
            }

            try {
                $quoteItem = $this->getQuoteItem($item, $product);
                if ($this->useShopgatePrices()) {
                    if ($this->taxData->priceIncludesTax($this->storeManager->getStore())) {
                        $quoteItem->setCustomPrice($amountGross);
                        $quoteItem->setOriginalCustomPrice($amountGross);
                    } else {
                        $quoteItem->setCustomPrice($amountNet);
                        $quoteItem->setOriginalCustomPrice($amountNet);
                    }
                }
                $quoteItem->setTaxPercent($item->getTaxPercent());

                if (!$item->isSimple()) {
                    $productWeight = $product->getTypeInstance()->getWeight($product);
                    $quoteItem->setWeight($productWeight);
                }

                $quoteItem->setRowWeight($quoteItem->getWeight() * $quoteItem->getQty());
                $this->quote->setItemsCount($this->quote->getItemsCount() + 1);
            } catch (\Exception $e) {
                $this->log->error(
                    "Error importing product to quote by id: {$product->getId()}, error: {$e->getMessage()}"
                );
                $this->log->error('SG item number: ' . $item->getItemNumber());
                $item->setMagentoError($e->getMessage());
            }
        }

        /**
         * Magento's flow is to save Quote on addItem, then on saveOrder load quote again. We mimic this here.
         */
        $this->quoteRepository->save($this->quote);
        $this->quote = $this->quoteRepository->get($this->quote->getId());
    }

    /**
     * @param Extended\OrderItem $item
     * @param MageProduct        $product
     *
     * @return bool | MageQuote\Item
     * @throws Exception
     * @throws ShopgateLibraryException
     * @throws \Exception
     * @throws LocalizedException
     */
    private function getQuoteItem(Extended\OrderItem $item, MageProduct &$product)
    {
        //todo-sg: move more of the buyInfo object to productHelper
        if (!$item->isSimple()) {
            $parent = $this->productHelper->loadById($item->getParentId());
            /** @var Configurable | Grouped $type */
            $type = $parent->getTypeInstance();

            if ($parent->getTypeId() === Configurable::TYPE_CODE) {
                $buyObject       = $this->productHelper->getBuyInfo($item, $parent);
                $superAttributes = $type->getConfigurableAttributesAsArray($parent);
                $superAttConfig  = [];

                foreach ($superAttributes as $productAttribute) {
                    $superAttConfig[$productAttribute['attribute_id']] = $product->getData(
                        $productAttribute['attribute_code']
                    );
                }
                $buyObject->setData('super_attribute', $superAttConfig);
                $product = $parent;
            } elseif ($parent->getTypeId() === Grouped::TYPE_CODE) {
                $product       = $this->productHelper->loadById($item->getChildId());
                $buyObject     = $this->productHelper->getBuyInfo($item, $product);
                $assocProducts = $type->getAssociatedProducts($parent);
                $buyObject->setData('super_group', $this->productHelper->getGrouped()->getSuperGroup($assocProducts));
                $buyObject->setData(
                    'super_product_config',
                    [
                        'product_type' => Grouped::TYPE_CODE,
                        'product_id'   => $parent->getId()
                    ]
                );
            } else {
                $buyObject = $this->productHelper->getBuyInfo($item, $product);
            }
        } else {
            $buyObject = $this->productHelper->getBuyInfo($item, $product);
        }

        $quoteItem = $this->quote->addProduct($product, $buyObject);
        if (is_string($quoteItem)) {
            if (in_array($quoteItem, $this->getValidationErrors())) {
                throw new Exception(new Phrase($quoteItem));
            } else {
                throw new ShopgateLibraryException(
                    ShopgateLibraryException::UNKNOWN_ERROR_CODE,
                    "Error on adding product to quote! Details: " . var_export($quoteItem, true),
                    true
                );
            }
        }

        return $this->quote->getItemByProduct($product);
    }

    /**
     * Magento option/input validation
     * error list
     *
     * @return array
     */
    public function getValidationErrors()
    {
        return [
            /** Bundle option related errors */
            __('The options you selected are not available.')->render(),
            __('The required options you selected are not available.')->render(),
            __('Please specify product option(s).')->render(),
            __('Please select all required options.')->render(),
            __('You need to choose options for your item.')->render(),

            /** Input field related errors */
            __('The text is too long')->render(),
        ];
    }

    /**
     * @todo-sg: create config for this at some point
     *
     * @return bool
     */
    private function useShopgatePrices()
    {
        return false;
    }

    /**
     * Assigns Shopgate cart customer to quote
     */
    protected function setCustomer()
    {
        $this->quoteCustomer->setEntity();
        $this->quoteCustomer->setAddress();
        $this->quote->save();
        $this->quoteCustomer->resetGuest();
    }
}
