<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
namespace Shopgate\Base\Model;

use Magento\Config\Model\ResourceModel\Config as ConfigResource;
use Magento\Framework\Api\SimpleDataObjectConverter;
use Magento\Framework\App\CacheInterface;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\App\RequestInterface;
use Magento\Store\Model\StoreManagerInterface;
use Shopgate\Base\Api\Config\CoreInterface;
use Shopgate\Base\Api\Config\SgCoreInterface;
use Shopgate\Base\Model\Utility\SgLoggerInterface;

include_once realpath(__DIR__) . '/../library/shopgate.php';

class Config extends \ShopgateConfig
{

    /** @var CoreInterface */
    protected $coreConfig;
    /** @var int */
    private $storeViewId;
    /** @var array - blacklist set via injection in di.xml */
    private $blacklistConfig = [];
    /** @var StoreManagerInterface */
    private $storeManager;
    /** @var SgLoggerInterface */
    private $logger;
    /** @var DirectoryList */
    private $directory;
    /** @var array - see global di.xml */
    private $configMapping;
    /** @var sgCoreInterface */
    private $sgCoreConfig;
    /** @var CacheInterface */
    private $cache;
    /** @var ConfigResource */
    private $configResource;
    /** @var array */
    private $configMethods;
    /** @var RequestInterface */
    private $request;
    /**
     * Will hold all the parameters loaded from DB, Library preset properties and DI
     *
     * @var array
     */
    private $configVars = [];

    /**
     * Assists with initializing
     *
     * @return bool
     */
    public function startup()
    {
        /** @var \Shopgate\Base\Helper\Initializer\Config $initializer */
        $manager              = ObjectManager::getInstance();
        $initializer          = $manager->get('Shopgate\Base\Helper\Initializer\Config');
        $this->storeManager   = $initializer->getStoreManager();
        $this->logger         = $initializer->getLogger();
        $this->directory      = $initializer->getDirectory();
        $this->sgCoreConfig   = $initializer->getSgCoreConfig();
        $this->coreConfig     = $initializer->getCoreConfig();
        $this->cache          = $initializer->getCache();
        $this->configResource = $initializer->getConfigResource();
        $this->configMapping  = $initializer->getConfigMapping();
        $this->configMethods  = $initializer->getConfigMethods();
        $this->request        = $initializer->getRequest();
        $this->plugin_name    = 'magento2';
        $this->loadArray($this->configMethods);

        return true;
    }

    /**
     * Prepares values to be saved
     *
     * @param array|null $settings
     */
    public function load(array $settings = null)
    {
        $classVars = array_keys(get_class_vars(get_class($this)));

        foreach ($settings as $name => $value) {
            if (in_array($name, $this->blacklistConfig)) {
                continue;
            }

            if (in_array($name, $classVars)) {
                $method = 'set' . SimpleDataObjectConverter::snakeCaseToUpperCamelCase($name);
                if (method_exists($this, $method)) {
                    $this->{$method}($this->castToType($value, $name));
                } else {
                    $this->logger->debug(
                        'The evaluated method "' . $method . '" is not available in class ' . __CLASS__
                    );
                }
            } else {
                if (array_key_exists($name, $this->additionalSettings)) {
                    $this->additionalSettings[$name] = $value;
                } else {
                    $this->logger->debug(
                        'The given setting property "' . $name . '" is not available in class ' . __CLASS__
                    );
                }
            }
        }
    }

    /**
     * Cast a given property value to the matching property type
     *
     * @param mixed  $value
     * @param string $property
     *
     * @return boolean|number|string|integer
     */
    protected function castToType($value, $property)
    {
        $type = $this->getPropertyType($property);

        switch ($type) {
            case 'array':
                return is_array($value) ? $value : explode(",", $value);
            case 'bool':
            case 'boolean':
                return (boolean) $value;
            case 'int':
            case 'integer':
                return (int) $value;
            case 'string':
                return (string) $value;
            default:
                return $value;
        }
    }

    /**
     * Fetches the property type described in phpdoc annotation
     *
     * @param string $property
     *
     * @return string
     */
    protected function getPropertyType($property)
    {
        if (!in_array($property, array_keys(get_class_vars('ShopgateConfig')))) {
            return 'string';
        }

        $r   = new \ReflectionProperty('ShopgateConfig', $property);
        $doc = $r->getDocComment();
        preg_match_all('#@var ([a-zA-Z-_]*(\[\])?)(.*?)\n#s', $doc, $annotations);

        $value = 'string';
        if (count($annotations) > 0 && isset($annotations[1][0])) {
            $value = $annotations[1][0];
        }

        return $value;
    }

    /**
     * Load general information and values
     */
    public function loadConfig()
    {
        $storeId = $this->sgCoreConfig->getStoreId($this->getShopNumber());
        $this->storeManager->setCurrentStore($storeId);
        $this->loadArray($this->toArray());
        $this->setExportTmpAndLogSettings();
    }

    /**
     * The Gods will forever hate me for using request interface here
     */
    public function getShopNumber()
    {
        if (is_null($this->shop_number)) {
            $shopNumber        = $this->request->getParam('shop_number');
            $item              = $this->sgCoreConfig->getShopNumberCollection($shopNumber)->getFirstItem();
            $this->shop_number = $item->getData('value') ? $shopNumber : '';
        }

        return $this->shop_number;
    }

    /**
     * Setup export, log and tmp folder and check if need to create them
     */
    protected function setExportTmpAndLogSettings()
    {
        $this->setExportFolderPath(
            $this->directory->getPath(DirectoryList::PUB) . DS . 'shopgate' . DS . $this->getShopNumber()
        );
        if (!file_exists($this->getExportFolderPath())) {
            @mkdir($this->getExportFolderPath(), 0777, true);
        }

        $this->setLogFolderPath(
            $this->directory->getPath(DirectoryList::LOG) . DS . 'shopgate' . DS . $this->getShopNumber()
        );
        if (!file_exists($this->getLogFolderPath())) {
            @mkdir($this->getLogFolderPath(), 0777, true);
        }

        $this->setCacheFolderPath(
            $this->directory->getPath(DirectoryList::TMP) . DS . 'shopgate' . DS . $this->getShopNumber()
        );
        if (!file_exists($this->getCacheFolderPath())) {
            @mkdir($this->getCacheFolderPath(), 0777, true);
        }
    }

    /**
     * @return array
     */
    public function toArray()
    {
        /**
         * Blacklist check
         */
        if (!empty($this->configVars)) {
            $temp = $this->configVars;
            foreach ($this->blacklistConfig as $key) {
                unset($temp[$key]);
            }

            return $temp;
        }

        $classProperties  = get_class_vars(get_class($this));
        $classVarMap      = $this->methodLoader($classProperties);
        $coreConfigMap    = $this->getCoreConfigMap($this->configMapping);
        $this->configVars = array_merge($classVarMap, $coreConfigMap);

        return $this->configVars;
    }

    /**
     * Uses the keys of the array as camelCase methods,
     * calls them and retrieves the data
     *
     * @param array $classProperties - where array('hello' => '') calls $this->getHello()
     *
     * @return array - where we get $return['hello'] = $this->getHello()
     */
    private function methodLoader(array $classProperties)
    {
        $result = [];
        foreach ($classProperties as $propertyName => $nullVal) {
            $result[$propertyName] = $this->getPropertyValue($propertyName);
        }

        return $result;
    }

    /** ======= SAVE LOGIC ========= */

    /**
     * Calls the properties getter method to retrieve the value
     *
     * @param $property
     *
     * @return null
     */
    private function getPropertyValue($property)
    {
        $value  = null;
        $getter = 'get' . SimpleDataObjectConverter::snakeCaseToUpperCamelCase($property);

        if (method_exists($this, $getter)) {
            $value = $this->{$getter}();
        } elseif ($this->returnAdditionalSetting($property)) {
            $value = $this->returnAdditionalSetting($property);
        }

        return $value;
    }

    /**
     * Traverses the config and retrieves data from
     * magento core_config_data table
     *
     * @param array $map - where key is Library defined property name
     *                   & value is core_config_data path
     *
     * @return array - $return['key'] = (cast type) value
     */
    private function getCoreConfigMap(array $map)
    {
        $result = [];

        foreach ($map as $key => $path) {
            $value = $this->coreConfig->getConfigByPath($path)->getData('value');
            if (!is_null($value)) {
                $result[$key] = $this->castToType($value, $key);
            }
        }

        return $result;
    }

    /**
     * Writes the given fields to magento
     *
     * @param array   $fieldList
     * @param boolean $validate
     *
     * @throws \Exception
     * @throws \ShopgateLibraryException
     */
    public function save(array $fieldList, $validate = true)
    {
        $this->logger->debug('# setSettings save start');

        if ($validate) {
            $this->validate($fieldList);
        }

        foreach ($fieldList as $property) {
            if (in_array($property, $this->blacklistConfig)) {
                continue;
            }

            if (isset($this->configMapping[$property])) {
                $config =
                    $this->sgCoreConfig->getSaveScope($this->configMapping[$property], $this->getShopNumber());
                $this->saveField(
                    $this->configMapping[$property],
                    $property,
                    $config->getScope(),
                    $config->getScopeId(),
                    null
                );
            }
        }

        $this->clearCache();
        $this->logger->debug('# setSettings save end');
    }

    /**
     * Saves this property to magento core_config_data
     *
     * @param string $path
     * @param string $property
     * @param string $scope
     * @param int    $scopeId
     * @param mixed  $value
     *
     * @throws \Exception
     */
    protected function saveField($path, $property, $scope, $scopeId, $value = null)
    {
        if (is_null($value)) {
            if (isset($this->configMapping[$property])) {
                $value = $this->getPropertyValue($property);
            } else {
                $this->logger->error('The specified property "' . $property . '" is not in the DI list');
            }
        }

        if (!is_null($value)) {
            $this->logger->debug(
                '    Saving config field \'' . $property . '\' with value \'' . $value . '\' to scope {\''
                . $scope . '\':\'' . $scopeId . '\'}'
            );
            $value = $this->prepareForDatabase($property, $value);
            $this->configResource->saveConfig($path, $value, $scope, $scopeId);
        }
    }

    /**
     * Converts values into a core_config_data compatible format
     *
     * @param string $property
     * @param mixed  $value
     *
     * @return mixed
     */
    protected function prepareForDatabase($property, $value)
    {
        $type = $this->getPropertyType($property);

        if ($type == 'array' && is_array($value)) {
            return implode(",", $value);
        }
        if (is_bool($value)) {
            $value = (int) $value;
        }

        return $value;
    }

    /** ======= Getters / Setters =========== */

    /**
     * Clears config cache after saving altered configuration
     */
    protected function clearCache()
    {
        $result = $this->cache->clean([\Magento\Framework\App\Config::CACHE_TAG]);
        $this->logger->debug(
            ' Config cache cleared with result: ' . ($result ? '[OK]' : '[ERROR]')
        );
    }

    /**
     * @return int
     */
    public function getStoreViewId()
    {
        if (!$this->storeViewId) {
            $this->storeViewId = $this->storeManager->getStore()->getId();
        }

        return $this->storeViewId;
    }

    /**
     * @param int $value
     */
    public function setStoreViewId($value)
    {
        $this->storeViewId = $value;
    }

    /**
     * @param array $keys
     */
    public function setBlacklistConfig($keys)
    {
        $this->blacklistConfig = $keys;
    }

    /**
     * todo-sg: must be implemented in MAGENTO2-6
     * di.xml controls which one it uses
     *
     * @see https://shopgate.atlassian.net/browse/MAGENTO2-6
     * @return string
     */
    public function getOauthAccessToken()
    {
        return null;
    }
}
