<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Model\Redirect\Route\Tags;

use Magento\Catalog\Model\Category;
use Magento\Catalog\Model\Product as MagentoProduct;
use Magento\Framework\Registry;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Tax\Model\Config as TaxConfig;
use Magento\Tax\Model\TaxCalculation;
use Shopgate\Base\Api\Config\CoreInterface;
use Shopgate\Base\Api\ExportInterface;
use Shopgate\Base\Model\Utility\SgLoggerInterface;
use Shopgate_Helper_Redirect_TagsGenerator as TagGenerator;

class Product extends Generic
{
    /** @var StoreManagerInterface */
    protected $storeManager;
    /** @var TaxConfig */
    protected $taxConfig;
    /** @var TaxConfig */
    protected $taxCalculation;
    /** @var SgLoggerInterface */
    private $logger;
    /** @var Registry */
    private $registry;

    /**
     * @inheritdoc
     *
     * @param TaxConfig         $taxConfig
     * @param TaxCalculation    $taxCalculation
     * @param SgLoggerInterface $logger
     * @param Registry          $registry
     */
    public function __construct(
        CoreInterface $config,
        StoreManagerInterface $storeManager,
        TaxConfig $taxConfig,
        TaxCalculation $taxCalculation,
        SgLoggerInterface $logger,
        Registry $registry
    ) {
        parent::__construct($config, $storeManager);
        $this->taxConfig      = $taxConfig;
        $this->taxCalculation = $taxCalculation;
        $this->logger         = $logger;
        $this->registry       = $registry;
    }

    /**
     * Generates page specific tags + generic tags
     *
     * @param string $pageTitle
     *
     * @return array
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function generate($pageTitle)
    {
        $tags    = parent::generate($pageTitle);
        $product = $this->registry->registry('current_product');

        if (!$product instanceof MagentoProduct) {
            $this->logger->error('Could not retrieve mage product from registry');

            return $tags;
        }
        /** @var \Magento\Store\Model\Store $store */
        $store           = $this->storeManager->getStore();
        $image           = $product->getMediaGalleryImages()->getFirstItem();
        $imageUrl        = is_object($image) ? $image->getData('url') : '';
        $name            = $product->getName();
        $description     = $product->getDescription();
        $availableText   = $product->isInStock() ? 'instock' : 'oos';
        $categoryName    = $this->getCategoryName();
        $defaultCurrency = $store->getCurrentCurrency()->getCode();

        $eanIdentifier = $this->config->getConfigByPath(ExportInterface::PATH_PROD_EAN_CODE)->getValue();
        $ean           = (string) $product->getData($eanIdentifier);

        $taxClassId   = $product->getTaxClassId();
        $storeId      = $store->getId();
        $taxRate      = $this->taxCalculation->getDefaultCalculatedRate($taxClassId, null, $storeId);
        $priceIsGross = $this->taxConfig->priceIncludesTax($this->storeManager->getStore());
        $price        = $product->getFinalPrice();
        $priceNet     = $priceIsGross ? round($price / (1 + $taxRate), 2) : round($price, 2);
        $priceGross   = !$priceIsGross ? round($price * (1 + $taxRate), 2) : round($price, 2);

        $productTags = [
            TagGenerator::SITE_PARAMETER_PRODUCT_IMAGE             => $imageUrl,
            TagGenerator::SITE_PARAMETER_PRODUCT_NAME              => $name,
            TagGenerator::SITE_PARAMETER_PRODUCT_DESCRIPTION_SHORT => $description,
            TagGenerator::SITE_PARAMETER_PRODUCT_EAN               => $ean,
            TagGenerator::SITE_PARAMETER_PRODUCT_AVAILABILITY      => $availableText,
            TagGenerator::SITE_PARAMETER_PRODUCT_CATEGORY          => $categoryName,
            TagGenerator::SITE_PARAMETER_PRODUCT_PRICE             => $priceGross,
            TagGenerator::SITE_PARAMETER_PRODUCT_PRETAX_PRICE      => $priceNet
        ];

        if ($priceGross || $priceNet) {
            $productTags[TagGenerator::SITE_PARAMETER_PRODUCT_CURRENCY]        = $defaultCurrency;
            $productTags[TagGenerator::SITE_PARAMETER_PRODUCT_PRETAX_CURRENCY] = $defaultCurrency;
        }
        $tags = array_merge($tags, $productTags);

        return $tags;
    }

    /**
     * Helps pulling category name from registry
     *
     * @return string
     */
    public function getCategoryName()
    {
        $category = $this->registry->registry('current_category');

        return ($category instanceof Category) ? $category->getName() : '';
    }
}
