<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Model\Shopgate\Extended;

/**
 * Class that combines both cart and order objects into one
 */
class Base extends \ShopgateOrder
{
    /** @var string */
    protected $internalCartInfo;
    /** @var OrderItemFactory */
    private $itemFactory;
    /** @var ExternalCouponFactory */
    private $externalCouponFactory;
    /** @var AddressFactory */
    private $addressFactory;

    /**
     * @inheritdoc
     */
    public function __construct(
        OrderItemFactory $itemFactory,
        ExternalCouponFactory $externalCouponFactory,
        AddressFactory $addressFactory,
        $data = []
    ) {
        $this->itemFactory           = $itemFactory;
        $this->externalCouponFactory = $externalCouponFactory;
        $this->addressFactory        = $addressFactory;

        return parent::__construct($data);
    }

    /**
     * Returns cart items with errors
     *
     * @return OrderItem[]
     */
    public function getItemsWithErrors()
    {
        $list = [];
        foreach ($this->getItems() as $item) {
            if ($item->getErrorCode()) {
                $list[] = $item;
            }
        }

        return $list;
    }

    /**
     * @return OrderItem[]
     */
    public function getItems()
    {
        return $this->items;
    }

    /**
     * Rewrite of the original just to
     * swap out the objects into our
     * custom item classes
     *
     * @param OrderItem[] | null $list
     */
    public function setItems($list)
    {
        if (!is_array($list)) {
            $this->items = null;

            return;
        }

        $items = [];
        foreach ($list as $index => $element) {
            if ((!is_object($element) || !($element instanceof \ShopgateOrderItem)) && !is_array($element)) {
                unset($list[$index]);
                continue;
            }

            if ($element instanceof \ShopgateOrderItem) {
                $element = $element->toArray();
            }

            $item = $this->itemFactory->create();
            $item->loadArray($element);
            $items[$item->getItemNumber()] = $item;
        }

        $this->items = $items;
    }

    /**
     * @param ExternalCoupon[] $list
     *
     * @throws \ShopgateLibraryException
     */
    public function setExternalCoupons($list)
    {
        if (!is_array($list)) {
            $this->external_coupons = null;

            return;
        }

        foreach ($list as $index => &$element) {
            if ((!is_object($element) || !($element instanceof \ShopgateExternalCoupon)) && !is_array($element)) {
                unset($list[$index]);
                continue;
            }

            if ($element instanceof \ShopgateExternalCoupon) {
                $coupon = $this->externalCouponFactory->create();
                $coupon->loadArray($element->toArray());
                $list[$index] = $coupon;
            } elseif (is_array($element)) {
                $coupon = $this->externalCouponFactory->create();
                $coupon->loadArray($element);
                $list[$index] = $coupon;
            }
        }

        $this->external_coupons = $list;
    }

    /**
     * @param int | string $id
     *
     * @return null|OrderItem
     */
    public function getItemById($id)
    {
        $items = $this->getItems();
        if (isset($items[$id])) {
            return $items[$id];
        }

        return null;
    }

    /**
     * @return ExternalCoupon[]
     */
    public function getExternalCoupons()
    {
        return $this->external_coupons;
    }

    /**
     * Checks if the customer in Cart is a guest
     * based on email or external id
     *
     * @return bool
     */
    public function isGuest()
    {
        return empty($this->getExternalCustomerId());
    }

    /**
     * @return string
     */
    public function getInternalCartInfo()
    {
        return $this->internalCartInfo;
    }

    /**
     * @param string $value
     */
    public function setInternalCartInfo($value)
    {
        $this->internalCartInfo = $value;
    }

    /**
     * @param array | \ShopgateAddress $value
     */
    public function setInvoiceAddress($value)
    {
        if (!is_object($value) && !($value instanceof \ShopgateAddress) && !is_array($value)) {
            $this->invoice_address = null;

            return;
        }

        if ($value instanceof \ShopgateAddress) {
            $newAddress = $this->addressFactory->create();
            $newAddress->loadArray($value->toArray());
            $value = $newAddress;
        } elseif (is_array($value)) {
            $newAddress = $this->addressFactory->create();
            $newAddress->loadArray($value);
            $newAddress->setIsDeliveryAddress(false);
            $newAddress->setIsInvoiceAddress(true);
            $value = $newAddress;
        }

        $this->invoice_address = $value;
    }

    /**
     * @param array | \ShopgateAddress $value
     */
    public function setDeliveryAddress($value)
    {
        if (!is_object($value) && !($value instanceof \ShopgateAddress) && !is_array($value)) {
            $this->delivery_address = null;

            return;
        }

        if ($value instanceof \ShopgateAddress) {
            $newAddress = $this->addressFactory->create();
            $newAddress->loadArray($value->toArray());
            $value = $newAddress;
        } elseif (is_array($value)) {
            $newAddress = $this->addressFactory->create();
            $newAddress->loadArray($value);
            $newAddress->setIsDeliveryAddress(true);
            $newAddress->setIsInvoiceAddress(false);
            $value = $newAddress;
        }

        $this->delivery_address = $value;
    }

    /**
     * @return Address
     */
    public function getInvoiceAddress()
    {
        return $this->invoice_address;
    }

    /**
     * @return Address
     */
    public function getDeliveryAddress()
    {
        return $this->delivery_address;
    }

    /**
     * Returns fields in array format ready
     * for magento import
     *
     * @return array
     */
    public function customFieldsToArray()
    {
        $result = [];
        foreach ($this->getCustomFields() as $field) {
            $result[$field->getInternalFieldName()] = $field->getValue();
        }

        return $result;
    }
}
