<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Model\Shopgate\Extended;

use Magento\GroupedProduct\Model\Product\Type\Grouped;
use Shopgate\Base\Helper\Quote;

class OrderItem extends \ShopgateOrderItem
{
    /** @var ItemInfo */
    protected $decodedInfo;
    /** @var ItemInfoFactory */
    private $itemInfoFactory;
    /** @var int $errorCode */
    private $errorCode;
    /** @var string $errorText */
    private $errorText;
    /** @var Quote */
    private $quoteHelper;

    /**
     * @param ItemInfoFactory $itemInfoFactory
     * @param Quote           $quoteHelper
     * @param array           $data
     */
    public function __construct(ItemInfoFactory $itemInfoFactory, Quote $quoteHelper, $data = [])
    {
        $this->itemInfoFactory = $itemInfoFactory;
        $this->quoteHelper     = $quoteHelper;
        parent::__construct($data);
    }

    /**
     * @inheritdoc
     */
    public function getUnitAmount()
    {
        return $this->adjustAmountByStackQty(
            parent::getUnitAmount()
        );
    }

    /**
     * @param float | string $amount
     *
     * @return float
     */
    private function adjustAmountByStackQty($amount)
    {
        $qty = $this->getStackQty();

        if ($qty > 1) {
            $amount = $amount / $qty;
        }

        return $amount;
    }

    /**
     * Retrieve stack qty if it exists
     *
     * @return int
     */
    public function getStackQty()
    {
        $info = $this->getInternalOrderInfo();
        $qty  = 1;

        if ($info->getStackQuantity() > 1) {
            $qty = $info->getStackQuantity();
        }

        return $qty;
    }

    /**
     * @return ItemInfo
     */
    public function getInternalOrderInfo()
    {
        if (!is_null($this->decodedInfo)) {
            return $this->decodedInfo;
        }

        $rawInfo = parent::getInternalOrderInfo();
        $obj     = $this->itemInfoFactory->create();

        return $this->decodedInfo = $obj->loadInfo($rawInfo);
    }

    /**
     * @inheritdoc
     */
    public function getUnitAmountWithTax()
    {
        return $this->adjustAmountByStackQty(
            parent::getUnitAmountWithTax()
        );
    }

    /**
     * Encodes to JSON if it's not empty or already JSON
     *
     * @param mixed $value
     *
     * @return string | null
     */
    public function setInternalOrderInfo($value)
    {
        if ($value instanceof ItemInfo) {
            $value = $value->toJson();
        } elseif (is_array($value)) {
            $value = \Zend_Json_Encoder::encode($value);
        }

        return parent::setInternalOrderInfo($value);
    }

    /**
     * Return parent ID if it exists
     *
     * @return string
     */
    public function getParentId()
    {
        if (!$this->isSimple()) {
            return explode('-', $this->getItemNumber())[0];
        }

        return $this->getItemNumber();
    }

    /**
     * Checks if current product is simple
     *
     * @return bool
     */
    public function isSimple()
    {
        return strpos($this->getItemNumber(), '-') === false;
    }

    /**
     * @return string | null
     */
    public function getChildId()
    {
        if (!$this->isSimple()) {
            return explode('-', $this->getItemNumber())[1];
        }

        return null;
    }

    /**
     * Helps unset the internal parameter
     */
    public function unsetDecodedInfo()
    {
        unset($this->decodedInfo);
    }

    /**
     * Takes a magento quote exception
     * and translated it into Shopgate error
     *
     * @param string $errorText
     *
     * @return OrderItem
     */
    public function setMagentoError($errorText)
    {
        $code            = $this->translateMagentoError($errorText);
        $message         = \ShopgateLibraryException::getMessageFor($code);
        $this->errorCode = $code;
        $this->errorText = $message;

        return $this;
    }

    /**
     * @param string $errorText
     *
     * @return int
     */
    private function translateMagentoError($errorText)
    {
        if (in_array($errorText, $this->quoteHelper->getValidationErrors())) {
            return \ShopgateLibraryException::CART_ITEM_INPUT_VALIDATION_FAILED;
        }

        return \ShopgateLibraryException::CART_ITEM_OUT_OF_STOCK;
    }

    /**
     * @return int
     */
    public function getErrorCode()
    {
        return $this->errorCode;
    }

    /**
     * @return string
     */
    public function getErrorText()
    {
        return $this->errorText;
    }

    /**
     * Uses an export ID based on expectations
     * of our backend API
     */
    public function getExportProductId()
    {
        return $this->getInternalOrderInfo()->getItemType() == Grouped::TYPE_CODE
            ? $this->getChildId()
            : $this->getParentId();
    }
}
