<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shoggate\Base\Test\Integration\Helper;

use Magento\Backend\App\Area\FrontNameResolver;
use Magento\Framework\Exception\LocalizedException;
use Shopgate\Base\Api\Config\SgCoreInterface;
use Shopgate\Base\Test\Bootstrap;
use Shopgate\Base\Test\Integration\Db\ConfigManager;

/**
 * @coversDefaultClass \Shopgate\Base\Helper\Redirect
 * @group Shopgate_Base_Redirect
 */
class RedirectTest extends \PHPUnit_Framework_TestCase
{
    /** @var \Shopgate\Base\Helper\Redirect */
    private $class;
    /** @var \Magento\Framework\Model\Context */
    private $context;
    /** @var \Magento\Framework\App\Request\Http */
    private $http;
    /** @var ConfigManager */
    private $cfgManager;

    public function setUp()
    {
        $manager          = Bootstrap::getObjectManager();
        $this->cfgManager = new ConfigManager;
        $this->context    = $manager->get('Magento\Framework\Model\Context');
        $this->http       = $manager->get('Magento\Framework\App\Request\Http');
        $this->class      = $manager->create('Shopgate\Base\Helper\Redirect');
    }

    /**
     * Sets up all methods so that isAllowed is true
     *
     * @covers ::isAllowed
     */
    public function testIsAllowedAllTrue()
    {
        $this->setAllowedTrue();

        $this->assertTrue($this->class->isAllowed());
    }

    /**
     * Helps set all the configs to get isAllowed to be true
     *
     * @throws LocalizedException
     */
    private function setAllowedTrue()
    {
        $this->context->getAppState()->setAreaCode('default');

        $this->http->setParam('ajax', false);
        $this->http->setParam('isAjax', false);
        $this->http->getHeaders()->addHeaderLine('USER_AGENT', 'flash');

        $this->cfgManager->setConfigValue(SgCoreInterface::PATH_ACTIVE, 1);
        $this->cfgManager->setConfigValue(SgCoreInterface::PATH_SHOP_NUMBER, 12345);
        $this->cfgManager->setConfigValue(SgCoreInterface::PATH_API_KEY, 11111);
        $this->cfgManager->setConfigValue(SgCoreInterface::PATH_CUSTOMER_NUMBER, 1234);
    }

    /**
     * Fails if in admin area
     *
     * @covers ::isAllowed
     * @throws LocalizedException
     */
    public function testIsAllowedAdminArea()
    {
        $this->setAllowedTrue();
        $this->context->getAppState()->setAreaCode(FrontNameResolver::AREA_CODE);

        $this->assertFalse($this->class->isAllowed());
    }

    /**
     * Fails if it's an XML request
     *
     * @covers ::isAllowed
     * @throws LocalizedException
     */
    public function testIsAllowedNotXmlRequest()
    {
        $this->setAllowedTrue();
        $this->http->getHeaders()->addHeaderLine('X_REQUESTED_WITH', 'XMLHttpRequest');

        $this->assertFalse($this->class->isAllowed());
    }

    /**
     * Fails if it's an ajax request
     *
     * @covers ::isAllowed
     * @throws LocalizedException
     */
    public function testIsAllowedAjaxRequest()
    {
        $this->setAllowedTrue();
        $this->http->setParam('ajax', true);

        $this->assertFalse($this->class->isAllowed());
    }

    /**
     * Fails if the module is deactivated
     *
     * @covers ::isAllowed
     * @throws LocalizedException
     */
    public function testIsAllowedNotActive()
    {
        $this->setAllowedTrue();
        $this->cfgManager->setConfigValue(SgCoreInterface::PATH_ACTIVE, 0);

        $this->assertFalse($this->class->isAllowed());
    }

    /**
     * Fails if there is no shop number set in config
     *
     * @covers ::isAllowed
     * @throws LocalizedException
     */
    public function testIsAllowedNoShopNumber()
    {
        $this->setAllowedTrue();
        $this->cfgManager->setConfigValue(SgCoreInterface::PATH_SHOP_NUMBER, 0);

        $this->assertFalse($this->class->isAllowed());
    }

    /**
     * Fails if there is no API key is set in config
     *
     * @covers ::isAllowed
     * @throws LocalizedException
     */
    public function testIsAllowedNoApiKey()
    {
        $this->setAllowedTrue();
        $this->cfgManager->setConfigValue(SgCoreInterface::PATH_API_KEY, null);

        $this->assertFalse($this->class->isAllowed());
    }

    /**
     * Fails if there is no customer number set in config
     *
     * @covers ::isAllowed
     * @throws LocalizedException
     */
    public function testIsAllowedNoCustomerNumber()
    {
        $this->setAllowedTrue();
        $this->cfgManager->setConfigValue(SgCoreInterface::PATH_CUSTOMER_NUMBER, null);

        $this->assertFalse($this->class->isAllowed());
    }

    /**
     * Asserts that the config is true if JS redirect is set
     * in the system > config
     *
     * @covers ::isTypeJavaScript
     */
    public function testIsTypeJavaScriptTrue()
    {
        $this->cfgManager->setConfigValue(SgCoreInterface::PATH_REDIRECT_TYPE, SgCoreInterface::VALUE_REDIRECT_JS);

        $this->assertTrue($this->class->isTypeJavaScript());
    }

    /**
     * Asserts that the config is false if HTTP redirect is set
     * in the system > config
     *
     * @covers ::isTypeJavaScript
     */
    public function testIsTypeJavaScriptFalse()
    {
        $this->cfgManager->setConfigValue(SgCoreInterface::PATH_REDIRECT_TYPE, SgCoreInterface::VALUE_REDIRECT_HTTP);

        $this->assertFalse($this->class->isTypeJavaScript());
    }

    /**
     * @after
     */
    public function tearDown()
    {
        $this->cfgManager->removeConfigs();
    }
}
