<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Export\Helper\Product;

use Magento\Catalog\Model\Product\Attribute\Source\Status;
use Magento\Catalog\Model\Product\Type;
use Magento\Catalog\Model\Product\Visibility;
use Magento\Catalog\Model\ProductFactory;
use Magento\ConfigurableProduct\Model\Product\Type\Configurable;
use Magento\GroupedProduct\Model\Product\Type\Grouped;
use Magento\Store\Model\StoreManagerInterface;
use Shopgate\Base\Model\Utility\SgLoggerInterface;
use Shopgate\Base\Model\Utility\SgProfiler;
use Shopgate\Export\Model\Export\Product;
use Shopgate\Export\Model\Export\ProductFactory as ExportFactory;

class Retriever
{
    /**  @const ALLOWED_PRODUCT_TYPES   Supported product types for export */
    const ALLOWED_PRODUCT_TYPES = [Type::TYPE_SIMPLE, Configurable::TYPE_CODE, Grouped::TYPE_CODE];
    /** @var SgLoggerInterface */
    private $log;
    /** @var  SgProfiler */
    private $profiler;
    /** @var StoreManagerInterface */
    private $storeManager;
    /** @var ProductFactory */
    private $productFactory;
    /** @var ExportFactory */
    private $exportFactory;

    /**
     * @param SgLoggerInterface     $logger
     * @param StoreManagerInterface $storeManager
     * @param ProductFactory        $productFactory
     * @param ExportFactory         $exportFactory
     * @param SgProfiler            $profiler
     * @param Configurable          $configurable
     *
     * @codeCoverageIgnore
     */
    public function __construct(
        SgLoggerInterface $logger,
        StoreManagerInterface $storeManager,
        ProductFactory $productFactory,
        ExportFactory $exportFactory,
        SgProfiler $profiler,
        Configurable $configurable
    ) {
        $this->log            = $logger;
        $this->storeManager   = $storeManager;
        $this->productFactory = $productFactory;
        $this->exportFactory  = $exportFactory;
        $this->profiler       = $profiler;
        $this->configurable   = $configurable;
    }

    /**
     * @param null | int $limit
     * @param null | int $offset
     * @param array      $uids
     *
     * @return array
     */
    public function getItems($limit = null, $offset = null, array $uids = [])
    {
        $this->log->access('Start Product Export...');
        $this->log->debug('Start Product Export...');
        $profiler = $this->profiler->start();
        $export   = [];

        /** @var \Magento\Catalog\Model\ResourceModel\Product\Collection $productCollection */
        $products          = $this->productFactory->create();
        $productCollection = $products->getCollection();
        $productCollection->addAttributeToFilter(
            'visibility',
            [
                Visibility::VISIBILITY_IN_CATALOG,
                Visibility::VISIBILITY_IN_SEARCH,
                Visibility::VISIBILITY_BOTH
            ]
        );
        $productCollection->addAttributeToFilter('status', Status::STATUS_ENABLED);
        $productCollection->addAttributeToFilter('type_id', ['in' => self::ALLOWED_PRODUCT_TYPES]);

        if (!empty($uids)) {
            $productCollection->addAttributeToFilter('entity_id', ['in' => $uids]);
        } elseif (!is_null($offset) && !is_null($offset)) {
            $productCollection->getSelect()->limit($limit, $offset);
            $this->log->debug("Product Export Limit: {$limit}");
            $this->log->debug("Product Export Offset: {$offset}");
        }

        foreach ($productCollection as $product) {
            $this->log->debug("Start Collection Product Load With ID: {$product->getId()}");
            $product = $this->productFactory->create()->load($product->getId());

            /** @var Product $productExportModel */
            $productExportModel = $this->exportFactory->create();
            try {
                if ($products->getTypeId() === Configurable::TYPE_CODE) {
                    $productExportModel->setItem($product);
                    $export[] = $productExportModel->generateData();
                } else {
                    $parentIds = $this->configurable
                        ->getParentIdsByChild($product->getId());
                    if (!empty($parentIds)) {
                        foreach ($parentIds as $parentId) {
                            $parentProduct = $this->productFactory
                                ->create()
                                ->load($parentId);

                            $productExportModel->setItem($parentProduct);
                            $export[] = $productExportModel->generateData();
                            $parentProduct->clearInstance();
                        }
                    }

                    $productExportModel->setItem($product);
                    $export[] = $productExportModel->generateData();
                }
                $product->clearInstance();
            } catch (\Exception $e) {
                $this->log->error(
                    "Skipping export of product with id: {$product->getId()}, message: " . $e->getMessage()
                );
            }
        }

        $profiler->end()->debug('Export Product duration %s seconds');
        $this->log->access('End Product Export...');
        $this->log->debug('End Product Export...');

        return $export;
    }
}
