<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Export\Helper;

use Magento\Catalog\Model\Product as MageProduct;
use Magento\ConfigurableProduct\Model\Product\Type\Configurable;
use Magento\Framework\Registry;
use Magento\Quote\Model\Quote as MageQuote;
use Magento\Quote\Model\QuoteRepository;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Tax\Helper\Data as MageTaxHelper;
use Shopgate\Base\Helper\Product\Type;
use Shopgate\Base\Helper\Product\Utility;
use Shopgate\Base\Helper\Quote\Coupon;
use Shopgate\Base\Helper\Quote\Customer;
use Shopgate\Base\Helper\Shopgate\CartItem;
use Shopgate\Base\Helper\Shopgate\ExternalCoupon;
use Shopgate\Base\Model\Shopgate\Extended\Base;
use Shopgate\Base\Model\Utility\SgLoggerInterface;
use Shopgate\Export\Helper\Tax as TaxHelper;

class Quote extends \Shopgate\Base\Helper\Quote
{
    /** @var Base */
    protected $sgBase;
    /** @var Type */
    private $type;
    /** @var CartItem */
    private $cartItemHelper;
    /** @var ExternalCoupon */
    private $couponHelper;
    /** @var TaxHelper */
    private $taxHelper;

    /**
     * @param Type                  $type
     * @param CartItem              $cartItemHelper
     * @param ExternalCoupon        $externalCoupon
     * @param MageQuote             $quote
     * @param Base                  $base
     * @param SgLoggerInterface     $logger
     * @param Utility               $productHelper
     * @param MageTaxHelper         $taxData
     * @param Customer              $quoteCustomer
     * @param Registry              $coreRegistry
     * @param StoreManagerInterface $storeManager
     * @param TaxHelper             $taxHelper
     * @param Coupon                $couponQuoteHelper
     * @param QuoteRepository       $quoteRepository
     */
    public function __construct(
        Type $type,
        CartItem $cartItemHelper,
        ExternalCoupon $externalCoupon,
        MageQuote $quote,
        Base $base,
        SgLoggerInterface $logger,
        Utility $productHelper,
        MageTaxHelper $taxData,
        Customer $quoteCustomer,
        Registry $coreRegistry,
        StoreManagerInterface $storeManager,
        TaxHelper $taxHelper,
        Coupon $couponQuoteHelper,
        QuoteRepository $quoteRepository
    ) {
        $this->type           = $type;
        $this->cartItemHelper = $cartItemHelper;
        $this->couponHelper   = $externalCoupon;
        $this->taxHelper      = $taxHelper;

        parent::__construct(
            $quote,
            $base,
            $logger,
            $productHelper,
            $taxData,
            $quoteCustomer,
            $coreRegistry,
            $storeManager,
            $couponQuoteHelper,
            $quoteRepository
        );
    }

    /**
     * @return \ShopgateExternalCoupon[]
     */
    public function getValidatedCoupons()
    {
        $coupons = [];
        foreach ($this->sgBase->getExternalCoupons() as $coupon) {
            if (!$coupon->getNotValidMessage()) {
                $amount = $this->quote->getSubtotal() - $this->quote->getSubtotalWithDiscount();
                $coupon->setIsValid(true);
                $coupon->setCurrency($this->quote->getStoreCurrencyCode());
                $coupon->setIsFreeShipping((bool) $this->quote->getShippingAddress()->getFreeShipping());
                $this->taxHelper->useClasses() ? $coupon->setAmountGross($amount) : $coupon->setAmountNet($amount);
            }
            $coupons[] = $this->couponHelper->dataToEntity($coupon->toArray());
        }

        return $coupons;
    }

    /**
     * @return \ShopgateCartItem[]
     * @throws \Exception
     */
    public function getValidatedItems()
    {
        $products = [];
        $this->quote->collectTotals();

        foreach ($this->quote->getAllVisibleItems() as $item) {
            $price   = $item->getProduct()->getFinalPrice();
            $percent = $item->getTaxPercent();

            if ($this->taxData->priceIncludesTax($this->quote->getStore())) {
                $priceInclTax = $price;
                $priceExclTax = $price / (1 + ($percent / 100));
            } else {
                $priceInclTax = $price * (1 + ($percent / 100));
                $priceExclTax = $price;
            }

            $stockData                    = $this->type->getType($item)->getStockData();
            $data['unit_amount']          = round($priceExclTax, 4);
            $data['unit_amount_with_tax'] = round($priceInclTax, 4);

            $productId = $item->getProductType() == Configurable::TYPE_CODE
                ? $item->getData('product_id') . '-' . $item->getOptionByCode('simple_product')->getProduct()->getId()
                : $item->getData('product_id');

            $product = $this->sgBase->getItemById($productId);
            if ($product) {
                $data['item_number'] = $productId;
                $data['options']     = $product->getOptions();
                $data['inputs']      = $product->getInputs();
                $data['attributes']  = $product->getAttributes();
            }

            $products[] = $this->cartItemHelper->dataToEntity(array_merge($data, $stockData));
        }

        /**
         * Export invalidated items
         */
        foreach ($this->sgBase->getItemsWithErrors() as $item) {
            $data     = $item->toArray();
            $cartItem = $this->cartItemHelper->dataToEntity($data);
            $cartItem->setError($item->getErrorCode());
            $cartItem->setErrorText($item->getErrorText());
            $products[] = $cartItem;
        }

        return $products;
    }
}
