<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Export\Test\Integration\Helper\Category;

use Magento\Catalog\Model\CategoryFactory;
use Magento\Store\Model\StoreManagerInterface;
use Shopgate\Base\Test\Bootstrap;
use Shopgate\Export\Helper\Category\Retriever;

/**
 * @coversDefaultClass \Shopgate\Export\Helper\Category\Retriever
 */
class RetrieverTest extends \PHPUnit_Framework_TestCase
{
    /** @var StoreManagerInterface */
    protected $storeManager;
    /** @var  Retriever */
    protected $class;
    /** @var array */
    protected $categories = [];
    /** @var CategoryFactory */
    protected $categoryFactory;

    /**
     * Init store manager + current class
     */
    public function setUp()
    {
        $objManager            = Bootstrap::getObjectManager();
        $this->storeManager    = $objManager->create('Magento\Store\Model\StoreManagerInterface');
        $this->class           = $objManager->create('Shopgate\Export\Helper\Category\Retriever');
        $this->categoryFactory = $objManager->create('Magento\Catalog\Model\CategoryFactory');
    }

    /**
     * Check if the created category can be pulled
     * individually by UID
     *
     * @covers ::buildCategoryTree
     * @covers ::getCategories
     */
    public function testGetCategoriesByUid()
    {
        $category = $this->createCategory();

        $sgCategories = $this->class->getCategories(null, null, [$category->getId()]);

        foreach ($sgCategories as $sgCategory) {
            /** @var \Shopgate\Export\Model\Export\Category $sgCategory */
            $this->assertEquals($category->getId(), $sgCategory->getUid());
        }
    }

    /**
     * Creates a category
     *
     * @return \Magento\Catalog\Model\Category
     * @throws \Exception
     */
    private function createCategory()
    {
        $rootId = $this->storeManager->getGroup()->getRootCategoryId();

        /** @var CategoryFactory $categoryFactory */
        $category = $this->categoryFactory->create();
        $category
            ->setIsActive(1)
            ->setName('Test' . rand(0, 999))
            ->setParentId($rootId)
            ->setPath('1/' . $rootId)
            ->setUrlPath('deep_link/url.html')
            ->setData('is_anchor', 1)
            ->setData('sort_order', '15');
        $this->categories[] = $category->save();

        return $category;
    }

    /**
     * @param $expected - expected number of items in collection
     * @param $limit
     * @param $offset
     *
     * @covers ::buildCategoryTree
     * @covers ::getCategories
     * @dataProvider limitProvider
     */
    public function testGetCategoriesLimit($expected, $limit, $offset)
    {
        if (is_null($expected)) {
            $rootId   = $this->storeManager->getGroup()->getRootCategoryId();
            $expected = $this
                ->categoryFactory
                ->create()
                ->getCollection()
                ->addFieldToFilter('path', ['like' => '%' . $rootId . '/%'])
                ->getSize();
        }
        $sgCategories = $this->class->getCategories($limit, $offset, []);

        $this->assertCount($expected, $sgCategories);
    }

    /**
     * Checks current export with online Schema
     *
     * @requires function fopen
     * @requires function libxml_set_external_entity_loader
     *
     * @uses     \DOMDocument::loadXML
     * @uses     \DOMDocument::schemaValidateSource
     *
     * @covers ::buildCategoryTree
     * @covers ::getCategories
     */
    public function testExportXml()
    {
        $category = $this->createCategory();
        $result   = $this->class->getCategories(null, null, [$category->getId()]);
        /** @var \Shopgate\Export\Model\Export\Category $sgCategory */
        $sgCategory = array_pop($result);
        $node       = new \Shopgate_Model_XmlResultObject('<categories></categories>');
        $xml        = $sgCategory->asXml($node)->asXML();
        $result     = $this->checkXmlFile($sgCategory->getXsdFileLocation(), $xml);

        $this->assertTrue($result, 'Category schema does not match what is exported');
    }

    /**
     * @param string $xsdFile    - online file location
     * @param string $currentXml - xml of the object
     *
     * @return bool
     */
    public function checkXmlFile($xsdFile, $currentXml)
    {
        $dom = new \DOMDocument();
        $dom->loadXML($currentXml);
        $schema = @file_get_contents($xsdFile);

        libxml_use_internal_errors(true);
        $result = @$dom->schemaValidateSource($schema);
        libxml_use_internal_errors(false);

        return $result;
    }

    /**
     * Provider for limits and offsets
     *
     * @return array
     */
    public function limitProvider()
    {
        return [
            'no limit test'             => [null, null, null],
            'first 5'                   => [5, 5, 0],
            'next 3'                    => [3, 3, 5],
            'no offset defaults to all' => [null, 5, null],
            'no limit default to all'   => [null, null, 0],
            'non-existing offset'       => [0, 5, 1000]
        ];
    }

    /**
     * Remove all created categories.
     * Area security is required to delete categories.
     */
    public function tearDown()
    {
        /**
         * @var \Magento\Framework\Registry $registry
         */
        $registry = Bootstrap::getObjectManager()->get('\Magento\Framework\Registry');
        $registry->register('isSecureArea', true, true);

        foreach ($this->categories as $category) {
            $category->delete();
        }
    }
}
