<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Export\Test\Integration\Helper\Product;

use Magento\Catalog\Model\Product\Visibility;
use Magento\Catalog\Model\ProductFactory;
use Magento\Store\Model\StoreManagerInterface;
use Shopgate\Base\Test\Bootstrap;
use Shopgate\Export\Helper\Product\Utility;

/**
 * @coversDefaultClass \Shopgate\Export\Helper\Product\Utility
 */
class HelperTest extends \PHPUnit_Framework_TestCase
{
    /** @var StoreManagerInterface */
    protected $storeManager;
    /** @var ProductFactory */
    protected $productFactory;
    /** @var Utility */
    protected $class;

    /**
     * Init store manager + current class
     */
    public function setUp()
    {
        $objManager           = Bootstrap::getObjectManager();
        $this->storeManager   = $objManager->create('Magento\Store\Model\StoreManagerInterface');
        $this->class          = $objManager->create('Shopgate\Export\Helper\Product\Utility');
        $this->productFactory = $objManager->create('Magento\Catalog\Model\ProductFactory');
    }

    /**
     * Provider for testVisibilitiesInCategories
     *
     * @return array
     */
    public function visibilitiesInCategoriesProvider()
    {
        return [
            'VISIBILITY_BOTH'        => [Visibility::VISIBILITY_BOTH, true],
            'VISIBILITY_IN_CATALOG'  => [Visibility::VISIBILITY_IN_CATALOG, true],
            'VISIBILITY_NOT_VISIBLE' => [Visibility::VISIBILITY_NOT_VISIBLE, false],
            'VISIBILITY_IN_SEARCH'   => [Visibility::VISIBILITY_IN_SEARCH, false]
        ];
    }

    /**
     * @param int  $visibility
     * @param bool $expectedResult
     *
     * @dataProvider visibilitiesInCategoriesProvider
     * @covers ::isVisibleInCategories
     */
    public function testVisibilitiesInCategories($visibility, $expectedResult)
    {
        $product = $this->productFactory->create()->load(1);
        $product->setVisibility($visibility);
        $this->assertEquals($expectedResult, $this->class->isVisibleInCategories($product));
    }

    /**
     * Provider for testSetVisibility
     */
    public function setVisibilityProvider()
    {
        return [
            'visibility level for visibility_both'        => [Visibility::VISIBILITY_BOTH, 'catalog_and_search'],
            'visibility level for visibility_in_catalog'  => [Visibility::VISIBILITY_IN_CATALOG, 'catalog'],
            'visibility level for visibility_not_visible' => [Visibility::VISIBILITY_NOT_VISIBLE, 'nothing'],
            'visibility level for visibility_in_search'   => [Visibility::VISIBILITY_IN_SEARCH, 'search'],
            'visibility level not defined visibility'     => ['Test', null]
        ];
    }

    /**
     * @param string $visibilityString
     * @param bool   $expectedResult
     *
     * @dataProvider setVisibilityProvider
     * @covers ::setVisibility
     */
    public function testSetVisibility($visibilityString, $expectedResult)
    {
        $product = $this->productFactory->create()->load(1);
        $product->setVisibility($visibilityString);
        $this->assertEquals($expectedResult, $this->class->setVisibility($product)->getLevel());
    }

    /**
     * @param string   $expectedLink
     * @param int|null $parentId
     * @param int      $childId
     * @param string   $visibility
     *
     * @covers ::getDeepLinkUrl
     * @covers       \Shopgate\Export\Model\Export\Product::setDeeplink
     *
     * @dataProvider getDeepLinkUrlProvider
     */
    public function testGetDeepLinkUrl($expectedLink, $parentId, $childId, $visibility)
    {
        $product = $this->productFactory->create()->load($childId);
        $product->setVisibility($visibility);

        if (!is_null($parentId)) {
            $parent = $this->productFactory->create()->load($parentId);
        } else {
            $parent = null;
        }

        $calculatedLink = $this->class->getDeepLinkUrl($product, $parent);
        $this->assertEquals($expectedLink, $calculatedLink);
    }

    /**
     * @return array (expected url, parent id, child id, visibility)
     */
    public function getDeepLinkUrlProvider()
    {
        return [
            'parent with invisible child' => [
                'http://localhost/index.php/strive-shoulder-pack.html',
                2,
                1,
                Visibility::VISIBILITY_NOT_VISIBLE
            ],
            'parent with visible child'   => [
                'http://localhost/index.php/joust-duffle-bag.html',
                2,
                1,
                Visibility::VISIBILITY_BOTH
            ],
            'plain simple without parent' => [
                'http://localhost/index.php/joust-duffle-bag.html',
                null,
                1,
                Visibility::VISIBILITY_BOTH
            ],
        ];
    }

    /**
     * Provider for testGetChildren
     */
    public function getChildrenProvider()
    {
        return [
            'try fetching children from config #83' => [83, 15],
            'try fetching children from simple #7'  => [7, 0],
            'try fetching children from grouped #2046' => [2046, 3]
        ];
    }

    /**
     * @param int  $productId
     * @param bool $expectedResult
     *
     * @covers ::getChildren
     * @covers       \Shopgate\Export\Model\Export\Product::setChildren
     *
     * @dataProvider getChildrenProvider
     */
    public function testGetChildren($productId, $expectedResult)
    {
        $product = $this->productFactory->create()->load($productId);
        $this->assertCount($expectedResult, $this->class->getChildren($product));
    }
}
